from PIL import Image
from .. import requirePackage


def is_svg(imgPath: str):
    """
    判断图片是否为svg格式

    check if the image is svg

    :param imgPath: 图片路径 | image path
    :return:
    """
    return imgPath.endswith(".svg") or imgPath.endswith('.svgz')


def is_eps(imgPath: str):
    """
    判断图片是否为eps格式

    check if the image is eps

    :param imgPath: 图片路径 | image path
    :return:
    """
    return imgPath.endswith(".eps") or imgPath.endswith('.epsi') or imgPath.endswith('.epsf')


def topng(imgPath: str):
    """
    将图片路径所指的图片转为jpg格式

    transform image to jpg

    :param imgPath: 图片路径 | image path
    :return:
    """
    if is_svg(imgPath):
        requirePackage("cairosvg", "svg2png")(
            url=imgPath, write_to=imgPath + ".png", dpi=300
        )
    elif is_eps(imgPath):
        requirePackage("wand.image", "Image")(filename=imgPath).save(filename=imgPath + ".png")
    else:
        Image.open(imgPath).save(imgPath + ".png", quality=100)


def tojpg(imgPath: str):
    """
    将图片路径所指的图片转为jpg格式

    transform image to jpg

    :param imgPath: 图片路径 | image path
    :return:
    """
    if is_svg(imgPath):
        imgPath = requirePackage("io", "BytesIO", "io")(
            requirePackage("cairosvg", "svg2png", "cairosvg")(url=imgPath, dpi=300)
        )
    elif is_eps(imgPath):
        imgPath = requirePackage("io", "BytesIO", "io")(
            requirePackage("wand.image", "Image", "wand.image")(filename=imgPath).make_blob()
        )
    Image.open(imgPath).convert("RGB").save(imgPath + ".jpg", quality=100)


def imgsConcat(imgs: list):
    """
    合并图片
    """

    from io import BytesIO
    from .. import (
        requirePackage,
        qs_default_console,
        qs_default_status,
        qs_error_string,
        qs_config,
    )

    Image = requirePackage("PIL", "Image", "Pillow")
    try:
        imgs = [Image.open(BytesIO(i)) for i in imgs if i]
    except:
        qs_default_console.print(qs_error_string, "样品图获取失败!")
        return

    if not imgs:
        qs_default_console.print(qs_error_string, "无样品图")
        return

    with qs_default_status("拼接图片中") as st:
        heights_len = min(len(imgs), 3)

        terminal_font_size = int(qs_config.basicSelect("terminal_font_size"))

        one_width = int(
            qs_default_console.width * terminal_font_size / heights_len / 2.125
        )

        heights = [0] * heights_len
        for i in imgs:
            one_height = int(one_width * i.size[1] / i.size[0])
            heights[heights.index(min(heights))] += one_height

        st.update("嗅探最佳拼接方式")
        max_height = qs_default_console.height * terminal_font_size

        while max(heights) > max_height and heights_len < len(imgs):
            heights_len += 1
            heights = [0] * heights_len
            one_width = int(
                qs_default_console.width / heights_len * terminal_font_size / 2.125
            )
            for i in imgs:
                one_height = int(one_width * i.size[1] / i.size[0])
                heights[heights.index(min(heights))] += one_height

        result = Image.new("RGBA", (one_width * heights_len, max(heights)))
        heights = [0] * heights_len

        imgs = [
            i.resize((one_width, int(one_width * i.size[1] / i.size[0]))) for i in imgs
        ]
        imgs = sorted(imgs, key=lambda i: -i.size[0] * i.size[1])

        for i in imgs:
            min_height_index = heights.index(min(heights))
            result.paste(i, (one_width * min_height_index, heights[min_height_index]))
            heights[min_height_index] += i.size[1]
    return result



from QuickProject import QproDefaultConsole as qs_default_console
from QuickProject import _ask, user_lang
from langsrc import LanguageDetector
import json
import sys
import os


platform = sys.platform
if platform.startswith("win"):
    dir_char = "\\"
else:
    dir_char = "/"


translate_engines = ["default", "TencentCloud", "DeepL", "DeepLX"]
lang_detector = LanguageDetector(user_lang, os.path.join(os.path.dirname(__file__), "lang.json"))

questions = {
    "default_currency": {
        "type": "input",
        "name": "default_currency",
        "message": lang_detector['ask_default_currency']
        + """

    CNY (Chinese)  USD (American) JPY (Japanese)  KRW (Korean)    DKK (Denmark)  EUR (Europe) 
    THB (Thailand) SAR (al-ummah) RUB (Russian)   BYR (Belarus)   RON (Romania)  PLN (Poland)
    BGN (Bulgaria) CZK (Czech)    ISK (Iceland)   VND (Vietnam)   DZD (Algeria)  ARS (Argentina)
    SEK (Sweden)   HUF (Hungary)  OMR (Amani)     AUD (Australia) MOP (Macao)    AED (United Arab Emirates)
    EGP (Egypt)    BHD (Bahrain)  BRL (Brazil)    HKD (HongKong)  COP (Colombia) PHP (Philippines)
    CAD (Canada)   KHR (Cambodia) QAR (Qatar)     HRK (Croatia)   KES (Kenya)    CRC (CostaRica)
    MXN (Mexico)   MAD (Morocco)  KWD (Kuwait)    LAK (Laos)      LBP (Lebanon)  MYR (Malaysia)
    BUK (Myanmar)  NOK (Norway)   SEK (Sweden)    RSD (Serbia)    TZS (Tanzania) ZAR (South Africa)
    BND (Brunei)   UGX (Uganda)   ZMK (Zambian)   SGD (Singapore) TWD (TaiWan)   LKR (Sri Lanka)  
    TRY (Turkey)   HUF (Hungary)  SYP (Syria)     IQD (Iraq)      INR (India)    NZD (New Zealand)
    GBP (England)  ILS (Israel)   JOD (Jordan)    CLP (Chile)     IDR (Indonesia)
    
  """
        + lang_detector['input_default_currency'],
        "validate": lambda val: val
        in {
            "CLP",
            "AED",
            "CZK",
            "THB",
            "MYR",
            "NZD",
            "LBP",
            "LAK",
            "HUF",
            "VND",
            "ZMK",
            "RSD",
            "CNH",
            "BYR",
            "HRK",
            "CHF",
            "CNY",
            "TWD",
            "CAD",
            "RON",
            "MOP",
            "CRC",
            "COP",
            "LKR",
            "IDR",
            "AUD",
            "ARS",
            "BGN",
            "KRW",
            "TZS",
            "JOD",
            "HKD",
            "EGP",
            "KHR",
            "ZAR",
            "BRL",
            "OMR",
            "BHD",
            "NOK",
            "PLN",
            "QAR",
            "RUB",
            "MAD",
            "EUR",
            "GBP",
            "BND",
            "SAR",
            "USD",
            "KWD",
            "SYP",
            "DKK",
            "ILS",
            "ISK",
            "DZD",
            "JPY",
            "SEK",
            "TRY",
            "INR",
            "KES",
            "SGD",
            "UGX",
            "PHP",
            "IQD",
            "BUK",
            "MXN",
        },
        "default": "USD",
    },
    "default_translate_engine": {
        "type": "list",
        "message": lang_detector['ask_translate_engine'],
        "choices": translate_engines,
        "default": "default",
    },
    "force_show_img": {
        "type": "confirm",
        "name": "force_show_img",
        "message": lang_detector['ask_force_show_img'],
        "default": "ITERM_SESSION_ID" in os.environ,
    },
    "default_proxy": {
        "type": "input",
        "name": "default_proxy",
        "message": lang_detector['ask_proxy'],
        "default": "Not set | 暂不设置",
    },
    "terminal_font_size": {
        "type": "input",
        "name": "terminal_font_size",
        "message": lang_detector['ask_font_size'],
        "default": "16",
    },
}


class QsConfig:
    def __init__(self, configPath):
        self.path = configPath
        if os.path.exists(configPath):
            try:
                with open(configPath, "r") as f:
                    self.config = json.loads(f.read())
            except:
                with open(configPath, "r", encoding="utf8") as f:
                    self.config = json.loads(f.read(), encoding="utf8")
        else:
            self.config = json.loads(
                """{
  "basic_settings": {
    "default_currency": "CNY",
    "default_translate_engine": {
      "index": 0,
      "support": ["default", "TencentCloud", "DeepL", "DeepLX"]
    },
    "default_proxy": "user:password@ip:port or ip:port",
    "force_show_img": false,
    "terminal_font_size": 16,
    "terminal_font_rate": 2.049
  },
  "API_settings": {
    "rmbg": "GET: https://www.remove.bg",
    "smms": "GET: https://sm.ms",
    "darksky": "GET: https://darksky.net/",
    "aliyun_oss_acid": "GET: https://www.aliyun.com/product/oss",
    "aliyun_oss_ackey": "GET: https://www.aliyun.com/product/oss",
    "aliyun_oss_bucket_url": "GET: https://www.aliyun.com/product/oss",
    "aliyun_oss_df_bucket": "GET: https://www.aliyun.com/product/oss",
    "txyun_scid": "GET: https://console.cloud.tencent.com/",
    "txyun_sckey": "GET: https://console.cloud.tencent.com/",
    "txyun_cos_df_bucket": "GET: https://console.cloud.tencent.com/",
    "txyun_df_region": "GET: ap-[location]",
    "qiniu_ac_key": "GET: http://qiniu.com/",
    "qiniu_sc_key": "GET: http://qiniu.com/",
    "qiniu_bk_name": "GET: [Qiniu Bucket Name]",
    "AipImageAPP_ID": "GET: https://cloud.baidu.com/product/imageprocess",
    "AipImageAPP_KEY": "GET: https://cloud.baidu.com/product/imageprocess",
    "AipImageSECRET_KEY": "GET: https://cloud.baidu.com/product/imageprocess",
    "AipNlpAPP_ID" : "GET: https://cloud.baidu.com/product/nlp_apply",
    "AipNlpAPP_KEY": "GET: https://cloud.baidu.com/product/nlp_apply",
    "AipNlpSECRET_KEY": "GET: https://cloud.baidu.com/product/nlp_apply",
    "alapi_token": "GET: https://user.alapi.cn/",
    "lolicon_token": "GET: https://api.lolicon.app/#/setu?id=apikey",
    "openai": "GET: https://openai.com/api/",
    "openai-email": "GET: https://openai.com/api/",
    "openai-password": "GET: https://openai.com/api/",
    "openai-paid": false,
    "openai-alapi": false,
    "DeepL": "GET: https://www.deepl.com/zh/pro-api?cta=header-pro-api/",
    "DeepLX": "GET: https://github.com/OwO-Network/DeepLX"
  }
}"""
            )
            self.config["basic_settings"]["default_currency"] = _ask(
                questions["default_currency"]
            )
            self.config["basic_settings"]["default_translate_engine"][
                "index"
            ] = translate_engines.index(_ask(questions["default_translate_engine"]))
            self.config["basic_settings"]["default_proxy"] = _ask(
                questions["default_proxy"]
            )
            self.update()
            qs_default_console.print(
                "\n" + lang_detector['config_store_in'],
                f"[bold green]{configPath}[/bold green]",
            )
            _ask(
                {
                    "type": "confirm",
                    "message": lang_detector['confirm'],
                    "default": True,
                }
            )
            if (
                platform.startswith("darwin")
                and "ITERM_SESSION_ID" not in os.environ
                and _ask(
                    {
                        "type": "confirm",
                        "message": lang_detector['recommand_using_iterm2'],
                        "default": True,
                    }
                )
            ):
                from .NetTools import open_url

                open_url("https://www.iterm2.com/")

    def update(self):
        with open(self.path, "w") as f:
            json.dump(self.config, f, indent=4, separators=(",", ": "))

    def basicSelect(self, key: str, default=None):
        if key not in self.config["basic_settings"]:
            if default is not None:
                self.config["basic_settings"][key] = default
                self.update()
            else:
                self.config["basic_settings"][key] = _ask(questions[key])
                self.update()
        return self.config["basic_settings"][key]

    def apiSelect(self, key):
        if key not in self.config["API_settings"]:
            raise KeyError
        return self.config["API_settings"].get(key, None)

    def basicUpdate(self, key: str, value: str):
        self.config["basic_settings"][key] = value
        return self.update()

    def apiUpdate(self, key: str, value: str):
        self.config["API_settings"][key] = value
        return self.update()



# coding=utf-8
"""
命令入口

Command entry
"""
import sys
from .funcList import *
from . import requirePackage


def qs_help(rep:str=""):
    """输出菜单 | Output menu"""
    if not rep or rep not in ['basic', 'system', 'net', 'api', 'image']:
        menu_show([
            ['basic', 'u, a, f, cal, time, pcat..'],
            ['system', 'top, \[mk/un]\[zip/tar/7z]..'],
            ['net', 'http, dl, netinfo, wifi...'],
            ['api', 'trans, smms, rmbg, loli...'],
            ['image', 'stbg, v2gif, v2mp3, v2mp4.']
        ])
        qs_default_console.print(f"\n[bold]{lang_detector['tutorial']}[/]\n", justify="center")
        qs_default_console.print("[bold magenta]qs lesson[/]", justify="center")
    else:
        menu_table(rep)


def main():
    """执行命令 | Execute"""
    debug_flag = "--qs-debug" in sys.argv
    if debug_flag:
        sys.argv.remove("--qs-debug")
    help_flag = '--help' in sys.argv
    if help_flag:
        sys.argv.remove('--help')
    if len(sys.argv) >= 2:
        try:
            func_name = sys.argv[1]
            if func_name not in cmd_table:
                qs_help(func_name)
            else:
                if help_flag:
                    return command_help(func_name)
                func_table = cmd_table[func_name]
                requirePackage(func_table["self"], func_table[func_name][0])()
        except Exception as e:
            from . import qs_default_console

            if debug_flag:
                qs_default_console.print_exception()
            else:
                from . import qs_error_string

                qs_default_console.log(qs_error_string, repr(e))
    else:
        qs_help()


if __name__ == "__main__":
    main()



"""
在终端预览图片 | 仅有MacOS下的iTerm2可用

preview image on terminal | Only iTerm2 is available,
"""
from .. import qs_default_console, qs_console_width, requirePackage, qs_default_status
from .. import _ask, user_lang, force_show_img, qs_config
import math
import base64
import sys
import os
import struct
import io

# 记录强制显示状态，避免重复询问
force_show_option = False
has_set_force_show_option = False

TMUX_WRAP_ST = b"\033Ptmux;"
TMUX_WRAP_ED = b"\033\\"

OSC = b"\033]"
CSI = b"\033["
ST = b"\a"  # \a = ^G (bell)


def get_image_shape(buf):
    """
    Extracts image shape as 2-tuple (width, height) from the content buffer.

    Supports GIF, PNG and other image types (e.g. JPEG) if PIL/Pillow is installed.
    Returns (None, None) if it can't be identified.
    """

    def _unpack(fmt, buffer, mode="Image"):
        try:
            return struct.unpack(fmt, buffer)
        except struct.error:
            raise ValueError("Invalid {} file".format(mode))

    # TODO: handle 'stream-like' data efficiently, not storing all the content into memory
    L = len(buf)

    if L >= 10 and buf[:6] in (b"GIF87a", b"GIF89a"):
        return _unpack("<hh", buf[6:10], mode="GIF")
    elif L >= 24 and buf.startswith(b"\211PNG\r\n\032\n") and buf[12:16] == b"IHDR":
        return _unpack(">LL", buf[16:24], mode="PNG")
    elif L >= 16 and buf.startswith(b"\211PNG\r\n\032\n"):
        return _unpack(">LL", buf[8:16], mode="PNG")
    else:
        # everything else: get width/height from PIL
        # TODO: it might be inefficient to write again the memory-loaded content to buffer...
        b = io.BytesIO()
        b.write(buf)

        try:
            im = requirePackage("PIL", "Image", "Pillow").open(b)
            return im.width, im.height
        except (IOError, OSError) as ex:
            # PIL.Image.open throws an error -- probably invalid byte input are given
            sys.stderr.write(
                "Warning: PIL cannot identify image; this may not be an image file"
                + "\n"
            )
        except ImportError:
            # PIL not available
            sys.stderr.write(
                "Warning: cannot determine the image size; please install Pillow" + "\n"
            )
            sys.stderr.flush()
        finally:
            b.close()

        return None, None


def _isinstance(obj, module, clsname):
    """A helper that works like isinstance(obj, module:clsname), but even when
    the module hasn't been imported or the type is not importable."""

    if module not in sys.modules:
        return False

    try:
        clstype = getattr(sys.modules[module], clsname)
        return isinstance(obj, clstype)
    except AttributeError:
        return False


def to_content_buf(data, fmt="png"):
    # TODO: handle 'stream-like' data efficiently, rather than storing into RAM

    if isinstance(data, bytes):
        return data

    elif isinstance(data, io.BufferedReader):
        buf = data
        return buf.read()

    elif isinstance(data, io.TextIOWrapper):
        return data.buffer.read()

    elif _isinstance(data, "numpy", "ndarray"):
        # numpy ndarray: convert to png
        im = data
        if len(im.shape) == 2:
            mode = "L"  # 8-bit pixels, grayscale
            im = im.astype(sys.modules["numpy"].uint8)
        elif len(im.shape) == 3 and im.shape[2] in (3, 4):
            mode = None  # RGB/RGBA
            if im.dtype.kind == "f":
                im = (im * 255).astype("uint8")
        else:
            raise ValueError(
                "Expected a 3D ndarray (RGB/RGBA image) or 2D (grayscale image), "
                "but given shape: {}".format(im.shape)
            )

        with io.BytesIO() as buf:
            # mode: https://pillow.readthedocs.io/en/4.2.x/handbook/concepts.html#concept-modes
            requirePackage("PIL", "Image", "Pillow").fromarray(im, mode=mode).save(
                buf, format=fmt
            )
            return buf.getvalue()

    elif _isinstance(data, "torch", "Tensor"):
        # pytorch tensor: convert to png
        from .. import requirePackage

        im = data

        with io.BytesIO() as buf:
            requirePackage("torchvision", "transforms").ToPILImage()(im).save(
                buf, format=fmt
            )
            return buf.getvalue()

    elif _isinstance(data, "tensorflow.python.framework.ops", "EagerTensor"):
        im = data
        return to_content_buf(im.numpy())

    elif _isinstance(data, "PIL.Image", "Image"):
        # PIL/Pillow images
        img = data

        with io.BytesIO() as buf:
            img.save(buf, format=fmt)
            return buf.getvalue()

    elif _isinstance(data, "matplotlib.figure", "Figure"):
        # matplotlib figures
        fig = data
        if fig.canvas is None:
            from matplotlib.backends.backend_agg import FigureCanvasAgg

            FigureCanvasAgg(fig)

        with io.BytesIO() as buf:
            fig.savefig(buf)
            return buf.getvalue()

    else:
        raise TypeError("Unsupported type : {}".format(type(data)))


def real_height(buf, pixels_per_line=qs_config.basicSelect("terminal_font_size")):
    _, im_height = get_image_shape(buf)
    if im_height:
        assert pixels_per_line > 0
        height = (im_height + (pixels_per_line - 1)) // pixels_per_line

        # automatically limit height to the current tty,
        # otherwise the image will be just erased
        try:
            return max(1, min(height, qs_default_console.height - 9))
        except OSError:
            # may not be a terminal
            pass
    else:
        # image height unavailable, fallback?
        return 10


def imgcat(
    buf,
    width_scale=None,
    height_scale=None,
    preserve_aspect_ratio=True,
    fp=None,
    force_show: bool = False,
):
    """
    Print image on terminal (iTerm2).

    Follows the file-transfer protocol of iTerm2 described at
    https://www.iterm2.com/documentation-images.html.

    Args:
        :param buf: the content of image in buffer interface, numpy array, etc.
        :param width: the width for displaying image, in number of characters (columns)
        :param height: the height for displaying image, in number of lines (rows)
        :param fp: The buffer to write to, defaults sys.stdout
        :param filename:
    """
    if fp is None:
        fp = sys.stdout.buffer  # for stdout, use buffer interface (py3)

    if len(buf) == 0:
        raise ValueError("Empty buffer")

    is_iterm2 = "ITERM_SESSION_ID" in os.environ

    if not is_iterm2 and not force_show:
        raise RuntimeError(
            "This function is only supported in iTerm2. "
            "Please set `force_show=True` to force show the image."
        )
    # now starts the iTerm2 file transfer protocol.
    fp.write(OSC)
    fp.write(b"1337;File=inline=1")
    fp.write(b";size=" + str(len(buf)).encode())
    if width_scale:
        fp.write(b";width=" + f'{width_scale}%%%%'.encode())
    if height_scale:
        fp.write(b";height=" + f'{height_scale}%%%%'.encode())
    if not preserve_aspect_ratio:
        fp.write(b";preserveAspectRatio=0")
    fp.write(b";inline=1")
    fp.write(b":")
    fp.flush()
    fp.write(base64.b64encode(buf))
    fp.write(ST)
    fp.write(b"\n")
    # flush is needed so that the cursor control sequence can take effect
    fp.flush()


def image_preview(
    img,
    is_url=False,
    set_proxy: str = "",
    set_referer: str = "",
    set_width_in_rc_file: int = 0,
    set_height_in_rc_file: int = 0,
    force_show: bool = force_show_img,
):
    """
    在终端预览图片 | 目前仅有MacOS下的iTerm可用, 但你可以开启强制显示选项预览

    preview image on terminal | At present, only iTerm under MacOS is available, but you can enable force show option to preview

    :param force_show: 强制显示图片，即使终端非iTerm2
    :param set_width_in_rc_file: 设置图片宽度，以像素为单位，默认为0，即不设置
    :param is_url: 是否为url
    :param img: opened file, numpy array, PIL.Image, matplotlib fig
    :param set_proxy: set proxy
    :param set_referer: set refer
    :param _fake_show: 返回图片比特流，用于自定义显示
    :return:
    """
    global force_show_option, has_set_force_show_option
    try:
        if not (
            force_show_option
            or force_show
            or (
                not has_set_force_show_option
                and _ask(
                    {
                        "type": "confirm",
                        "message": "Preview image on this terminal?"
                        if user_lang != "zh"
                        else "确认在该终端预览图片?",
                        "default": False,
                    }
                )
            )
        ):
            has_set_force_show_option = True
            return
        else:
            has_set_force_show_option = True
            force_show_option = True
        if not is_url and (isinstance(img, str) and not os.path.exists(img)):
            is_url = img.startswith("http")

        if is_url:
            if img_bytes := requirePackage(".NetTools.NormalDL", "normal_dl")(
                img,
                set_referer=set_referer,
                set_proxy=set_proxy,
                disableStatus=True,
                write_to_memory=True,
            ):
                if img.endswith('.svg'):
                    img_bytes = requirePackage("cairosvg", "svg2png")(bytestring=img_bytes)
                img = requirePackage("PIL", "Image", "Pillow").open(
                    requirePackage("io", "BytesIO")(img_bytes)
                )
            else:
                return qs_default_console.print(
                    requirePackage('.', 'qs_error_string'),
                    "Failed to download image" if user_lang != "zh" else "图片下载失败",
                )
        elif isinstance(img, str) and os.path.exists(img):
            if img.endswith('.svg') or img.endswith('.svgz'):
                qs_default_console.print(requirePackage('.', 'qs_warning_string'), 'Convert svg to png (dpi=300) ...' if user_lang != 'zh' else 'svg将转换为png (dpi=300) ...')
                img = requirePackage("io", "BytesIO")(requirePackage("cairosvg", "svg2png")(url=img, dpi=300))
            elif img.endswith('.eps') or img.endswith('.epsf') or img.endswith('.epsi'):
                qs_default_console.print(requirePackage('.', 'qs_warning_string'), 'Convert eps to png (dpi=300) ...' if user_lang != 'zh' else 'eps将转换为png (dpi=300) ...')
                img = requirePackage("io", "BytesIO")(requirePackage("wand.image", "Image")(filename=img).make_blob())
            img = requirePackage("PIL", "Image", "Pillow").open(img)
        else:
            qs_default_console.print(
                requirePackage('.', 'qs_warning_string'),
                '将自适应解析图片格式' if user_lang != 'zh' else 'Will adaptively parse the image format',
            )

        _st = qs_default_status.status

        qs_default_status(
            "Loading image" if user_lang != "zh" else "正在加载图片",
        ).start()

        buf = to_content_buf(img)

        qs_default_status("Calculating the position of the image" if user_lang != "zh" else "计算图片摆放位置")

        width, height = get_image_shape(buf)
        console_width = (
            qs_console_width if not set_width_in_rc_file else set_width_in_rc_file
        )
        console_height = qs_default_console.height if not set_height_in_rc_file else set_height_in_rc_file

        max_width_scale = set_width_in_rc_file / qs_console_width if set_width_in_rc_file else 1
        max_height_scale = set_height_in_rc_file / qs_default_console.height if set_height_in_rc_file else 1
        max_width_scale *= max_height_scale
        
        rate = width / height
        _real_height = math.ceil(height / qs_config.basicSelect("terminal_font_size"))
        _real_width = math.ceil(
            width
            * qs_config.basicSelect("terminal_font_rate", 2.049)
            / qs_config.basicSelect("terminal_font_size")
        )

        if _real_width > console_width:
            height_scale = console_width / _real_width
            _real_height = math.floor(_real_height * height_scale)
            _real_width = console_width
        if _real_height > console_height:
            width_scale = console_height / _real_height
            _real_width = math.floor(_real_width * width_scale)

        qs_default_status.stop()
        if rate > 1:
            imgcat(
                buf,
                width_scale=max_width_scale * 100,
                force_show=force_show_option,
            )
        else:
            qs_default_console.print(
                " " * math.floor((qs_console_width - _real_width) / 2),
                end="",
            )
            imgcat(
                buf,
                height_scale=max_height_scale * 100,
                force_show=force_show_option,
            )

        if _st:
            qs_default_status.start()
    except Exception:
        qs_default_status.stop()
        qs_default_console.print_exception()
        return



from .. import requirePackage, user_lang, qs_default_console, qs_error_string
from . import pre_check
import time

lang_table = {
    'zh': 'Chinese',
    'en': 'English',
    'fra': 'French',
    'ru': 'Russian',
    'spa': 'Spanish',
    'ara': 'Arabic',
}

API_KEY = pre_check("openai", ext=False)
ALAPI = pre_check("openai-alapi", ext=False)
if ALAPI:
    import requests
    import json


_chatbot = None


class AlapiChatbot:
    def __init__(
            self, 
            system_prompt=f"You are ChatGPT, a large language model trained by OpenAI. Respond conversationally with Markdown format and using {lang_table[user_lang]} Language.",
            no_record: bool = False
        ):
        from .alapi import alapi_token, v2_url

        self.no_record = no_record
        self.api_key = alapi_token
        self.url = v2_url
        self.messages = [
            {
                "role": "system",
                "content": system_prompt,
            }
        ]

    def ask(self, prompt):
        self.messages.append(
            {
                "role": "user",
                "content": prompt,
            }
        )
        while res := (
            requests.post(
                self.url + "chatgpt/pro",
                json={
                    "token": self.api_key,
                    "message": self.messages,
                },
            )
            .json()
        ):
            if res['code'] == 422 or not res.get('data'):
                qs_default_console.print(qs_error_string, res)
                time.sleep(3)
                continue
            else:
                break
        res = res.get("data").get("content")
        if not self.no_record:
            self.messages.append(
                {
                    "role": "assistant",
                    "content": res,
                }
            )
        return res

    def ask_stream(self, prompt):  # not support now
        self.messages.append(
            {
                "role": "user",
                "content": prompt,
            }
        )
        post_stream = requests.post(
            self.url + "chatgpt/stream",
            json={
                "token": self.api_key,
                "message": self.messages,
            },
            stream=True,
        )
        total_res = ""
        for line in post_stream.iter_lines(decode_unicode=True):
            line = line.strip()
            if not line:
                continue
            res = json.loads(line)
            if res['success']:
                total_res = res['text']
                yield total_res

        if not self.no_record:
            self.messages.append(
                {
                    "role": "assistant",
                    "content": total_res,
                }
            )


def create_bot(no_record: bool = False):
    if ALAPI:
        return AlapiChatbot(no_record=no_record)
    else:
        return (
            requirePackage("revChatGPT.V3", "Chatbot", "revChatGPT")(
                API_KEY,
                system_prompt=f"You are ChatGPT, a large language model trained by OpenAI. Respond conversationally with Markdown format and using {lang_table[user_lang]} Language.",
            )
            if API_KEY
            else requirePackage("revChatGPT.V1", "Chatbot", "revChatGPT")(
                {
                    "email": pre_check("openai-email"),
                    "password": pre_check("openai-password"),
                    "paid": pre_check("openai-paid"),
                }
            )
        )


def chatGPT(
    prompt: str,
    wait_all: bool = False,
    no_record: bool = False,
):
    """
    使用OpenAI的GPT-3 API进行聊天

    :param prompt: 聊天内容
    :param wait_all: 是否等待所有内容
    :param no_record: 是否不记录聊天内容
    """
    global _chatbot
    if _chatbot is None:
        _chatbot = create_bot(no_record=no_record)
    if wait_all:
        return _chatbot.ask(prompt)
    else:
        return _chatbot.ask_stream(prompt)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['QuickStart_Rhy',
 'QuickStart_Rhy.API',
 'QuickStart_Rhy.ImageTools',
 'QuickStart_Rhy.NetTools',
 'QuickStart_Rhy.NumbaTools',
 'QuickStart_Rhy.SystemTools',
 'QuickStart_Rhy.ThreadTools',
 'QuickStart_Rhy.TuiTools',
 'QuickStart_Rhy.Wrapper']

package_data = \
{'': ['*']}

modules = \
['lang']
install_requires = \
['Qpro>=0.12.1,<0.13.0',
 'inquirer-rhy>=0.1.2,<0.2.0',
 'requests>=2.31.0,<3.0.0',
 'rich>=13.3.4,<14.0.0',
 'urllib3>=1.26.15,<2.0.0']

entry_points = \
{'console_scripts': ['qs = QuickStart_Rhy.main:main']}

setup_kwargs = {
    'name': 'quickstart-rhy',
    'version': '0.7.5',
    'description': 'A Command Line Toolbox',
    'long_description': '# QuickStart_Rhy\n\n```shell\npip3 install quickstart-rhy                                         # stable version\npip3 install git+https://github.com/Rhythmicc/quickstart-rhy.git -U # beta version but more features\n```\n\n| Key  | Value                                              |\n| :--: | -------------------------------------------------- |\n| ENV  | **^Python 3.7**                                    |\n| FONT | **Cascadia Code / Meslo**                          |\n| DOCS | <https://rhythmlian.cn/2020/02/14/QuickStart-Rhy/> |\n\n## Tab Complete\n\n1. fig\n\n   ```shell\n   npx @fig/publish-spec -p complete/fig/qs-<your language>.ts --name qs\n   ```\n\n2. zsh\n\n   ```shell\n   mv _qs /path/in/$FPATH/\n   ```\n',
    'author': 'Rhythmicc',
    'author_email': 'rhythmlian.cn@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
